/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 16/05/2007
   derniere modification : 21/07/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECore.h>
#include <SCE/interface/SCEModel.h>

/* modele binde pour le rendu */
static SCE_SModel *binded = NULL;


int SCE_Init_Model (void)
{
    return SCE_OK;
}
void SCE_Quit_Model (void)
{
}

static unsigned int SCE_Model_DefaultGetLODFunc (float);
static void SCE_Model_DefaultUpdateLODFunc (SCE_SModel*);

/* ajoute le 14/07/2008 */
static void SCE_Model_InitStates (SCE_SModelStates *states)
{
    states->actived = SCE_TRUE;
    states->cullface = SCE_TRUE;  /* backface culling active par defaut */
    states->depthtest = SCE_TRUE; /* depth test active par defaut */
    states->alphatest = SCE_TRUE; /* alpha test active par defaut */
}

/* revise le 21/07/2008 */
void SCE_Model_Init (SCE_SModel *mdl)
{
    mdl->meshs = NULL;
    mdl->n_meshs = 0;
    mdl->lod = 0;
    mdl->use_lod = SCE_FALSE;
    mdl->getlodfunc = SCE_Model_DefaultGetLODFunc;
    mdl->updatelodfunc = SCE_Model_DefaultUpdateLODFunc;

    mdl->mat = NULL;

    mdl->node = NULL;
    SCE_BoundingBox_Init (&mdl->box);
    mdl->have_box = SCE_FALSE;

    mdl->render = SCE_Model_RenderFunc;
    mdl->render_param = NULL;

    SCE_Model_InitStates (&mdl->states);

    mdl->userdata = NULL;
}


/* revise le 02/03/2008 */
SCE_SModel* SCE_Model_Create (void)
{
    SCE_SModel *mdl = NULL;

    SCE_btstart ();
    mdl = SCE_malloc (sizeof *mdl);
    if (!mdl)
        Logger_LogSrc ();
    else
    {
        SCE_Model_Init (mdl);
        if (!(mdl->mat = SCE_Material_Create ()))
        {
            SCE_Model_Delete (mdl);
            Logger_LogSrc ();
            SCE_btend ();
            return NULL;
        }
        if (!(mdl->node = SCE_Node_Create ()))
        {
            SCE_Model_Delete (mdl), mdl = NULL;
            Logger_LogSrc ();
        }
    }
    SCE_btend ();
    return mdl;
}

/* revise le 02/03/2008 */
void SCE_Model_Delete (SCE_SModel *mdl)
{
    SCE_btstart ();
    if (mdl)
    {
        SCE_Mesh_DeleteList (mdl->meshs);
        SCE_Material_Delete (mdl->mat);
        SCE_Node_Delete (mdl->node);
        SCE_free (mdl);
    }
    SCE_btend ();
}

/* ajoute le 14/04/2008 */
void SCE_Model_SetData (SCE_SModel *mdl, void *data)
{
    mdl->userdata = data;
}
void* SCE_Model_GetData (SCE_SModel *mdl)
{
    return mdl->userdata;
}

/* ajoute le 24/03/2008 */
void SCE_Model_SetMeshs (SCE_SModel *mdl, SCE_SMesh **meshs, int num)
{
    mdl->n_meshs = 0;
    SCE_Mesh_DeleteList (mdl->meshs);
    mdl->meshs = meshs;
    if (num > 0)
        mdl->n_meshs = num;
    else 
        while (meshs[mdl->n_meshs])
            mdl->n_meshs++;
}

/* ajoute le 25/06/2007 */
/* revise le 02/03/2008 */
int SCE_Model_LoadMesh (SCE_SModel *mdl, const char *fname)
{
    SCE_SMesh **meshs = NULL;

    SCE_btstart ();
    meshs = SCE_Mesh_Load (fname, NULL);
    if (!meshs)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_Model_SetMeshs (mdl, meshs, 0);
    SCE_btend ();
    return SCE_OK;
}

/* ajoute le 02/04/2008 */
void SCE_Model_SetRenderFunc (SCE_SModel *mdl, SCE_FModelRenderFunc f, void *p)
{
    mdl->render = f;
    mdl->render_param = p;
}
/* ajoute le 07/04/2008 */
void SCE_Model_SetDefaultRenderFunc (SCE_SModel *mdl)
{
    mdl->render = SCE_Model_RenderFunc;
    mdl->render_param = NULL;
}
/* ajoute le 07/04/2008 */
int SCE_Model_HaveRenderFunc (SCE_SModel *mdl)
{
    /* on repere si c'est la fonction par defaut qui est en place */
    return ((mdl->render == SCE_Model_RenderFunc) ? SCE_FALSE : SCE_TRUE);
}

/* ajoute le 02/03/2008 */
void SCE_Model_ActivateBackfaceCulling (SCE_SModel *mdl, int actived)
{
    mdl->states.cullface = actived;
}
void SCE_Model_EnableBackfaceCulling (SCE_SModel *mdl)
{
    mdl->states.cullface = SCE_TRUE;
}
void SCE_Model_DisableBackfaceCulling (SCE_SModel *mdl)
{
    mdl->states.cullface = SCE_FALSE;
}

/* ajoute le 05/07/2008 */
void SCE_Model_Activate (SCE_SModel *mdl, int active)
{
    mdl->states.actived = active;
}
void SCE_Model_Enable (SCE_SModel *mdl)
{
    mdl->states.actived = SCE_TRUE;
}
void SCE_Model_Disable (SCE_SModel *mdl)
{
    mdl->states.actived = SCE_FALSE;
}
/* ajoute le 05/07/2008 */
int SCE_Model_IsActived (SCE_SModel *mdl)
{
    return mdl->states.actived;
}

/* ajoute le 02/03/2008 */
/* revise le 13/07/2008 */
void SCE_Model_ActivateLOD (SCE_SModel *mdl, int use)
{
    mdl->use_lod = use;
}
void SCE_Model_EnableLOD (SCE_SModel *mdl)
{
    mdl->use_lod = SCE_TRUE;
}
void SCE_Model_DisableLOD (SCE_SModel *mdl)
{
    mdl->use_lod = SCE_FALSE;
}

/* ajoute le 14/07/2008 */
void SCE_Model_SetGetLODFunc (SCE_SModel *mdl, SCE_FModelGetLODFunc func)
{
    if (func)
        mdl->getlodfunc = func;
    else
        mdl->getlodfunc = SCE_Model_DefaultGetLODFunc;
}

/* ajoute le 21/07/2008 */
void SCE_Model_SetUpdateLODFunc (SCE_SModel *mdl, SCE_FModelUpdateLODFunc func)
{
    if (func)
        mdl->updatelodfunc = func;
    else
        mdl->updatelodfunc = SCE_Model_DefaultUpdateLODFunc;
}


/* ajoute le 14/07/2008 */
static unsigned int SCE_Model_DefaultGetLODFunc (float diagonal)
{
    return (unsigned int)(1.4142/diagonal - 1.0);
}

/* ajoute le 21/07/2008 */
static void SCE_Model_DefaultUpdateLODFunc (SCE_SModel *mdl)
{
    /* TODO: fake contribution culling hohoho :-' */
    if (mdl->size <= 0.157) /* what's this fucking number ??? :---' */
        SCE_Model_Disable (mdl);
    else
        SCE_Model_Enable (mdl);
}

/* ajoute le 14/07/2008 */
static float SCE_Model_GetBBDiagonalLength (SCE_SModel *mdl, SCE_SCamera *cam)
{
    float *points;
    SCE_TVector4 vertex1, vertex2;
    SCE_TMatrix4 matrix;
    float dist;

    /* recuperation de la position de la camera */
    SCE_Matrix4_Copy (matrix, SCE_Camera_GetFinalView (cam));
    SCE_Matrix4_Inverse (matrix);
    SCE_Matrix4_GetTranslation (matrix, vertex1);
    /* recuperation de la position du modele */
    SCE_Matrix4_GetTranslation (SCE_Node_GetFinalMatrix(mdl->node),vertex2);
    /* calcul de la distance entre les deux points */
    dist = SCE_Vector3_Distance (vertex1, vertex2);

    /* recuperation de deux points de la bounding box du modele */
    /* TODO: faire attention, box peut etre en mode "push" */
    points = SCE_BoundingBox_GetPoints (&mdl->box);
    /* voir SCEBoundingBox.c pour l'emplacement des points 0 et 2 */
    SCE_Vector3_Copy (vertex1, &points[0]);
    SCE_Vector3_Copy (vertex2, &points[2]);
    /* ajustement de la distance */
    dist += SCE_BoundingBox_GetDepth (&mdl->box)/2;
    vertex1[2] -= dist; vertex1[3] = 1.0;
    vertex2[2] -= dist; vertex2[3] = 1.0;
    /* projection des points via la matrice de projection */
    SCE_Matrix4_MulV4 (SCE_Camera_GetProj (cam), vertex1);
    SCE_Matrix4_MulV4 (SCE_Camera_GetProj (cam), vertex2);
    /* 2D suffit, on n'a pas besoin de la profondeur */
    SCE_Vector2_Operator1 (vertex1, /=, vertex1[3]);
    SCE_Vector2_Operator1 (vertex2, /=, vertex2[3]);
    /* calcul de la distance entre les deux points projetes */
    return SCE_Vector3_Distance (vertex1, vertex2);
}

/* ajoute le 02/03/2008 */
/* revise le 21/07/2008 */
void SCE_Model_UpdateLOD (SCE_SModel *mdl, SCE_SCamera *cam)
{
    if (mdl->use_lod)
    {
        if (!mdl->have_box)
        {
#ifdef SCE_DEBUG
            Logger_PrintMsg ("can't compute LOD level : never "
                             "bounding box on this model");
#endif
            return;
        }

        mdl->size = SCE_Model_GetBBDiagonalLength (mdl, cam);
        mdl->lod = mdl->getlodfunc (mdl->size);
        mdl->updatelodfunc (mdl);
    }
    else
        mdl->lod = 0;
}

/* ajoute le 05/03/2008 */
unsigned int SCE_Model_GetLOD (SCE_SModel *mdl)
{
    return mdl->lod;
}
/* ajoute le 02/03/2008 */
SCE_SNode* SCE_Model_GetNode (SCE_SModel *mdl)
{
    return mdl->node;
}
/* ajoute le 02/03/2008 */
SCE_SBoundingBox* SCE_Model_GetBoundingBox (SCE_SModel *mdl)
{
    return &mdl->box;
}
/* ajoute le 02/03/2008 */
SCE_SMesh** SCE_Model_GetMeshs (SCE_SModel *mdl)
{
    return mdl->meshs;
}
/* ajoute le 02/03/2008 */
SCE_SMesh* SCE_Model_GetMesh (SCE_SModel *mdl)
{
    return mdl->meshs[((mdl->lod >= mdl->n_meshs) ? mdl->n_meshs-1 : mdl->lod)];
}
/* ajoute le 02/03/2008 */
SCE_SMaterial* SCE_Model_GetMaterial (SCE_SModel *mdl)
{
    return mdl->mat;
}

/* ajoute le 28/03/2008 */
int SCE_Model_HaveBoundingBox (SCE_SModel *mdl)
{
    return mdl->have_box;
}
/* ajoute le 02/03/2008 */
int SCE_Model_GenerateBoundingBox (SCE_SModel *mdl)
{
    SCE_SBoundingBox box;

    SCE_btstart ();
    SCE_BoundingBox_Init (&box);
    if (SCE_Mesh_GenerateBoundingBox (mdl->meshs[0/*..?*/], &box) < 0)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    mdl->box = box;
    mdl->have_box = SCE_TRUE;
    SCE_btend ();
    return SCE_OK;
}

/* ajoute le 14/07/2008 */
int SCE_Model_SortFaces (SCE_SModel *mdl, int order, SCE_SCamera *cam)
{
    SCE_TMatrix4 matrix;  /* matrice finale de la position de
                             la camera par rapport au modele */
    SCE_TMatrix4 nodemat; /* matrice du noeud de l'objet */
    SCE_TVector3 campos;  /* position finale de la camera */

    /* copie des matrices */
    SCE_Matrix4_Copy (matrix, SCE_Camera_GetFinalView (cam));
    SCE_Matrix4_Copy (nodemat, SCE_Node_GetFinalMatrix (mdl->node));

    /* calcul de la position de la camera par rapport au modele en supposant
       que la position et l'orientation du modele soient nulles */
    SCE_Matrix4_Inverse (matrix);
    SCE_Matrix4_Inverse (nodemat);
    SCE_Matrix4_MulMul (matrix, nodemat);
    /* recuperation de la position */
    SCE_Matrix4_GetTranslation (matrix, campos);

    /* execution de l'algorithme de tri */
    return SCE_Mesh_SortFaces (SCE_Model_GetMesh (mdl), order, campos);
}

/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_Bind (SCE_SModel *mdl)
{
    binded = mdl;
    if (mdl)
    {
        SCE_Mesh_Use (SCE_Model_GetMesh (mdl));
        SCE_Material_Use (mdl->mat);
    }
    else
    {
        SCE_Material_Use (NULL);
        SCE_Mesh_Use (NULL);
    }
}

/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_BindMesh (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Mesh_Use (SCE_Model_GetMesh (mdl));
    else
        SCE_Mesh_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_BindMaterial (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Material_Use (mdl->mat);
    else
        SCE_Material_Use (NULL);
}

/* ajoute le 04/03/2008 */
void SCE_Model_BindNode (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Node_Use (mdl->node);
    else
        SCE_Node_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_Draw (void)
{
    SCE_Mesh_Draw ();
}

/* ajoute le 07/04/2008 */
void SCE_Model_RenderFunc (SCE_SModel *mdl, void *param)
{
    SCE_CSetState (GL_CULL_FACE, mdl->states.cullface);
    SCE_CSetState (GL_DEPTH_TEST, mdl->states.depthtest);
    SCE_CSetState (GL_ALPHA_TEST, mdl->states.alphatest);

    SCE_CSetActiveMatrix (SCE_MAT_MODELVIEW);
    SCE_CPushMatrix ();
    SCE_Node_Use (mdl->node); /* _D'ABORD_ on set la matrice (hack moche btw) */
    SCE_Material_Use (mdl->mat);
    SCE_Mesh_Render (SCE_Model_GetMesh (mdl));
    SCE_CPopMatrix ();
}

/* revise le 02/03/2008 */
void SCE_Model_Render (SCE_SModel *mdl)
{
    mdl->render (mdl, mdl->render_param);
}
