/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 11 mars 2007
   derniere modification le 19/08/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEMath.h>
#include <SCE/utils/SCEMatrix.h>
#include <SCE/utils/SCEString.h>
#include <SCE/utils/SCEMedia.h>
#include <SCE/utils/SCEResources.h>

#include <SCE/core/SCECSupport.h>
#include <SCE/core/SCECMatrix.h>

#include <SCE/interface/SCEQuad.h>
#include <SCE/interface/SCETexture.h>

/**
 * \file SCETexture.c
 * \copydoc texture
 * 
 * \file SCETexture.h
 * \copydoc texture
 */

/**
 * \defgroup texture High-level textures and render textures managment
 * \ingroup interface
 * \internal
 * \brief Unify all texture's types and render textures in one module to provide
 * an user-friendly textures managment, this module offers a simplified
 * interface to use jointly framebuffer and coretexture managers
 */

/** @{ */

/* ajoute le 14/06/2007 */
static int sce_tex_enabled = SCE_TRUE; /* booleen, true : textures activees */

/* ajoute le 10/07/2007 */
static SCE_STexture *binded = NULL;

/* ajoute le 07/10/2007 */
static SCE_STexture *used = NULL;
static int used_unit = 0;


/* revise le 06/10/2007 */
int SCE_Init_Texture (void)
{
    return SCE_OK;
}

/* revise le 06/10/2007 */
void SCE_Quit_Texture (void)
{
}

/* ajoute le 06/10/2007 */
void SCE_Texture_Bind (SCE_STexture *tex)
{
    binded = tex;
    /* NOTE: binding de la texture du coeur ? */
}

/* ajoute le 22/07/2008 */
static void SCE_Texture_Init (SCE_STexture *tex)
{
    unsigned int i;
    for (i=0; i<6; i++)
        tex->fb[i] = NULL;
    for (i=0; i<SCE_MAX_ATTACHMENT_BUFFERS; i++)
    {
        tex->tex[i] = NULL;
        tex->canfree[i] = SCE_FALSE;
    }
    tex->ct = 0;
    tex->type = 0;
}

/* ajoute le 20/07/2008 */
static void SCE_Texture_SetupParameters (SCE_STexture *tex)
{
    int type = tex->type;
    /* assignation de quelques parametres pour les textures cubemap */
    if (type == SCE_RENDER_COLOR_CUBE || type == SCE_TEX_CUBE ||
        type == SCE_RENDER_DEPTH_CUBE)
    {
        SCE_CBindTexture (tex->tex[tex->ct]);
        SCE_CSetTextureParam_ (GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        SCE_CSetTextureParam_ (GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        SCE_CSetTextureParam_ (GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
        SCE_CBindTexture (NULL);
        /* on remet la precedente texture active */
        SCE_Texture_Use (used, used_unit);
    }
}

/* ajoute le 22/07/2008 */
static int SCE_Texture_MakeRender (SCE_STexture *tex, int type, int w, int h)
{
    tex->fb[0] = SCE_CCreateFramebuffer ();
    if (!tex->fb[0])
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    SCE_CBindFramebuffer (tex->fb[0]);
    /* s'il s'agit d'une color, on ajoute un depth buffer */
    if (type == SCE_COLOR_BUFFER)
        SCE_CAddRenderBuffer_ (SCE_DEPTH_BUFFER, 0, w, h);

    if (SCE_CCreateRenderTexture_ (type, 0, 0, 0, w, h) < 0)
    {
        SCE_Texture_Delete (tex);
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    /* le proprietaire de la texture reste le frame buffer
       car tex->canfree[n] est egal a SCE_FALSE par defaut */
    tex->tex[tex->ct] = SCE_CGetRenderTexture_ (type);
    SCE_CBindFramebuffer (NULL);
    return SCE_OK;
}

/* ajoute le 22/07/2008 */
static int SCE_Texture_MakeRenderCube(SCE_STexture *tex, int type, int w, int h)
{
    unsigned int i = 0;
    SCE_CTexData data;

    tex->tex[tex->ct] = SCE_CCreateTexture (SCE_TEX_CUBE);
    if (!tex->tex[tex->ct])
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    for (i=0; i<6; i++)
    {
        tex->fb[i] = SCE_CCreateFramebuffer ();
        if (!tex->fb[i])
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }

        SCE_CInitTexData (&data);

        /* le target est defini dans AddTextureTexData */
        data.w = w;
        data.h = h;
        data.type = SCE_UNSIGNED_BYTE;

        if (type == SCE_DEPTH_BUFFER)
        {
            data.fmt = GL_DEPTH_COMPONENT;
            data.pxf = GL_DEPTH_COMPONENT24;
        }
        else
            data.fmt = data.pxf = GL_RGBA;

        if (SCE_CAddTextureTexDataDup (tex->tex[tex->ct], SCE_TEX_POSX + i,
                                       &data) < 0)
        {
            Logger_LogSrc ();
            SCE_btend ();
            return SCE_ERROR;
        }

        /* s'il s'agit d'une color, on ajoute un depth buffer */
        if (type == SCE_COLOR_BUFFER)
            SCE_CAddRenderBuffer (tex->fb[i], SCE_DEPTH_BUFFER, 0, w, h);
                
        SCE_CAddRenderTexture (tex->fb[i], type, SCE_TEX_POSX + i,
                               tex->tex[tex->ct], 0, SCE_FALSE);
    }
    return SCE_OK;
}

/* revise le 22/07/2008 */
/**
 * \brief Creates a new texture and/or a new render texture
 * \param type defines the type of the texture to create, can be SCE_TEX_1D,
 * SCE_TEX_2D, SCE_TEX_3D or SCE_TEX_CUBE to make a simple texture, to make
 * a render texture, you may use the following types : SCE_RENDER_COLOR,
 * SCE_RENDER_COLOR_CUBE, SCE_RENDER_DEPTH or SCE_RENDER_DEPTH_CUBE
 * \param w the width of the new texture
 * \param h the height of the new texture
 * \returns the new texture
 */
SCE_STexture* SCE_Texture_Create (SCEenum type, int w, int h)
{
    SCE_STexture *tex = NULL;

    SCE_btstart ();
    tex = SCE_malloc (sizeof *tex);
    if (!tex)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    SCE_Texture_Init (tex);
    tex->type = type;

    switch (type)
    {
    case SCE_RENDER_DEPTH:
        type = SCE_DEPTH_BUFFER;
    case SCE_RENDER_COLOR:
        if (type == SCE_RENDER_COLOR)
            type = SCE_COLOR_BUFFER;
        /* construction de la texture */
        if (SCE_Texture_MakeRender (tex, type, w, h) < 0)
        {
            SCE_Texture_Delete (tex);
            Logger_LogSrc ();
            SCE_btend ();
            return NULL;
        }
        break;

    case SCE_RENDER_DEPTH_CUBE:
        type = SCE_DEPTH_BUFFER;
    case SCE_RENDER_COLOR_CUBE:
        if (type == SCE_RENDER_COLOR_CUBE)
            type = SCE_COLOR_BUFFER;
        /* construction de la texture */
        if (SCE_Texture_MakeRenderCube (tex, type, w, h) < 0)
        {
            SCE_Texture_Delete (tex);
            Logger_LogSrc ();
            SCE_btend ();
            return NULL;
        }
        break;

    default:
        /* aucun type "render" specifie, c'est une texture normale */
        tex->tex[tex->ct] = SCE_CCreateTexture (type);
        if (!tex->tex[tex->ct])
        {
            SCE_free (tex);
            Logger_LogSrc ();
            SCE_btend ();
            return NULL;
        }
    }

    SCE_Texture_SetupParameters (tex);

    SCE_btend ();
    return tex;
}


/* ... */
#define SCE_DEFAULTFUNC(action)\
SCE_STexture *back = binded;\
SCE_Texture_Bind (tex);\
action;\
SCE_Texture_Bind (back);

#define SCE_DEFAULTFUNCR(t, action)\
t r;\
SCE_STexture *back = binded;\
SCE_Texture_Bind (tex);\
r = action;\
SCE_Texture_Bind (back);\
return r;

/* ajoute le 07/10/2007 */
/**
 * \brief Deletes an existing texture
 * \param tex the texture to delete
 */
void SCE_Texture_Delete (SCE_STexture *tex)
{
    SCE_DEFAULTFUNC (SCE_Texture_Delete_ ())
}
/* revise le 07/10/2007 */
void SCE_Texture_Delete_ (void)
{
    SCE_btstart ();
    if (binded)
    {
        unsigned int i;
        for (i=0; i<6; i++)
            SCE_CDeleteFramebuffer (binded->fb[i]);
        for (i=0; i<SCE_MAX_ATTACHMENT_BUFFERS; i++)
        {
            if (binded->canfree[i])
                SCE_CDeleteTexture (binded->tex[i]);
        }
        SCE_free (binded);
        binded = NULL;
    }
    SCE_btend ();
}

/* ajoute le 05/04/2008 */
/**
 * \brief Gets the \p tex 's type that has been given to SCE_Texture_Create()
 */
int SCE_Texture_GetType (SCE_STexture *tex)
{
    return tex->type;
}
/* ajoute le 05/04/2008 */
/**
 * \brief Gets the type of the core texture used by \p tex
 * this function calls SCE_CGetTextureTarget()
 */
int SCE_Texture_GetCType (SCE_STexture *tex)
{
    return SCE_CGetTextureTarget (tex->tex[tex->ct]);
}

/* ajoute le 07/10/2007 */
/**
 * \brief Gets the core texture used by \p tex
 */
SCE_CTexture* SCE_Texture_GetCTexture (SCE_STexture *tex)
{
    return tex->tex[tex->ct];
}
/* ajoute le 07/10/2007 */
SCE_CTexture* SCE_Texture_GetCTexture_ (void)
{
    return binded->tex[binded->ct];
}

/**
 * \brief Gets the width of a texture
 *
 * This function calls SCE_CGetTextureWidth (SCE_Texture_GetCTexture (\p tex),
 * \p target, \p level)
 * \sa SCE_CGetTextureWidth()
 */
int SCE_Texture_GetWidth (SCE_STexture *tex, int target, int level)
{
    return SCE_CGetTextureWidth (tex->tex[tex->ct], target, level);
}
/**
 * \brief Gets the height of a texture
 *
 * This function calls SCE_CGetTextureHeight (SCE_Texture_GetCTexture (\p tex),
 * \p target, \p level)
 * \sa SCE_CGetTextureHeight()
 */
int SCE_Texture_GetHeight (SCE_STexture *tex, int target, int level)
{
    return SCE_CGetTextureHeight (tex->tex[tex->ct], target, level);
}


/* ajoute le 07/10/2007 */
/**
 * \brief Builds a texture created by SCE_Texture_Create(), you must build
 * your textures before any use of them.
 * \param tex the texture to build
 * \param use_mipmap set at SCE_TRUE, the texture will use mipmapping
 * \returns SCE_ERROR on error, SCE_OK otherwise
 */
int SCE_Texture_Build (SCE_STexture *tex, int use_mipmap)
{
    int hw_mipmap = 0;

    SCE_btstart ();
    SCE_CBindTexture (tex->tex[tex->ct]);

    /* par defaut, le moteur tente de faire generer les mipmaps en hardware */
    hw_mipmap = (use_mipmap && SCE_CHasCap (SCE_TEX_HW_GEN_MIPMAP));
    if (SCE_CGetTextureTarget (tex->tex[tex->ct]) != SCE_TEX_CUBE)
        hw_mipmap = (SCE_CGetTextureNumMipmaps_ (0) <= 1 && hw_mipmap);

    /* construction */
    if (SCE_CBuildTexture (tex->tex[tex->ct], use_mipmap, hw_mipmap) < 0)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    SCE_CBindTexture (NULL);

    SCE_btend ();
    return SCE_OK;
}
/* revise le 07/10/2007 */
int SCE_Texture_Build_ (int use_mipmap)
{
    return SCE_Texture_Build (binded, use_mipmap);
}

/* revise le 07/10/2007 */
/**
 * \brief After any change on your texture, this function will apply yours
 * modifications manpower on your texture
 * \returns SCE_ERROR on error, SCE_OK otherwise
 */
int SCE_Texture_Update (SCE_STexture *tex)
{
    SCE_btstart ();
    if (SCE_CUpdateTexture (tex->tex[tex->ct], -1, -1) < 0)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    SCE_btend ();
    return SCE_OK;
}
/* ajoute le 07/10/2007 */
int SCE_Texture_Update_ (void)
{
    return SCE_Texture_Update (binded);
}

/* ajoute le 19/08/2008 */
/**
 * \brief Sets an OpenGL texture's parameter
 * \param tex the texture to set the parameter
 * \param id the render target's identifier, used only for multiple render
 * targets, if you don't added any texture by calling
 * SCE_Texture_AddRenderTexture() or SCE_Texture_AddRenderCTexture(),
 * ignore this parameter and set it at 0
 * \param pname second glTexParameteri parameter
 * \param param third glTexParameteri parameter
 */
void SCE_Texture_SetParam (SCE_STexture *tex, unsigned int id,
                           SCEenum pname, SCEint param)
{
    SCE_CSetTextureParam (tex->tex[id], pname, param);
}
/**
 * \brief Float version of SCE_Texture_SetParam()
 */
void SCE_Texture_SetParamf (SCE_STexture *tex, unsigned int id,
                            SCEenum pname, SCEfloat param)
{
    SCE_CSetTextureParamf (tex->tex[id], pname, param);
}

/* ajoute le 13/01/2008 */
/* revise le 20/07/2008 */
/**
 * \brief Loads and creates a new texture from one or more files
 * \param type force the texture's type to load, if SCE_TEX_CUBE is specified,
 * six parameters are expected in \p args defining respectively posx, negx,
 * posy, negy, posz and negz cube face's image
 * \param w force the new texture's width (0 to keep unchanged)
 * \param h force the new texture's height (0 to keep unchanged)
 * \param d force the new texture's depth (0 to keep unchanged)
 * \param args the va_list that gives the file names
 * \returns the new texture
 */
SCE_STexture* SCE_Texture_LoadArg (int type, int w, int h, int d, va_list args)
{
    SCE_STexture *tex = NULL;

    SCE_btstart ();
    tex = SCE_Texture_Create (type, w, h);
    if (!tex)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    /*< TODO: ! temporaire ! */
    SCE_CDeleteTexture (tex->tex[tex->ct]);

    tex->tex[tex->ct] = SCE_CLoadTextureArg (type, w, h, d, args);
    if (!tex->tex[tex->ct])
    {
        SCE_Texture_Delete (tex);
        Logger_LogSrc ();
        tex = NULL;
    }
    else
    {
        SCE_Texture_SetupParameters (tex);
        tex->canfree[tex->ct] = SCE_TRUE;
    }

    SCE_btend ();
    return tex;
}
/* ajoute le 07/10/2007 */
/* revise le 13/01/2008 */
/**
 * \brief Loads and creates a new texture by calling SCE_Texture_LoadArg()
 * with the given variables parameters
 * 
 * \see SCE_Texture_LoadArg()
 */
SCE_STexture* SCE_Texture_Load (int type, int w, int h, int d, ...)
{
    va_list args;
    SCE_STexture *tex = NULL;
    va_start (args, d);
    tex = SCE_Texture_LoadArg (type, w, h, d, args);
    va_end (args);
    return tex;
}

/* ajoute le 22/07/2008 */
/**
 * \brief Adds a texture as a new render target's
 * \param tex the texture to add a new render texture
 * \param id render target's identifier (SCE_COLOR_BUFFERn, SCE_DEPTH_BUFFER
 * or SCE_STENCIL_BUFFER)
 * \param ctex the texture to add as a render target (is a core texture)
 * \param canfree boolean which indicates if \p tex has right to delete \p ctex
 * \returns SCE_ERROR on error, SCE_OK otherwise
 * \note If \p ctex has different dimensions or pixel format of \p tex,
 * then this function maybe generate an error that is implementation and/or
 * hardware dependent
 * \sa SCE_CAddRenderTexture()
 *
 * This function requires \p tex is already a render buffer texture
 * (its type must be SCE_RENDER_*). After adding a new render texture target,
 * SCEngine will use the Multiple Render Targets OpenGL's extension, if you
 * don't support it, SCE_ERROR is returned and \p ctex is not added. This
 * function calls SCE_CAddRenderTexture() under the frame buffer of \p tex.
 */
int SCE_Texture_AddRenderCTexture (SCE_STexture *tex, int id,
                                   SCE_CTexture *ctex, int canfree)
{
    SCEenum target;
    unsigned int i = 0;

    target = SCE_CGetTextureTarget (tex->tex[0]);
    if (target != SCE_CGetTextureTarget (ctex))
    {
#ifdef SCE_DEBUG
        Logger_Log (SCE_INVALID_ARG);
        Logger_LogMsg ("you can't add a render texture "
                       "with a different target");
#endif
        return SCE_ERROR;
    }

    /* ajout de la texture au framebuffer */
    if (target == SCE_TEX_CUBE)
        for (i=0; i<6; i++)
        {
            if (SCE_CAddRenderTexture (tex->fb[i], id, SCE_TEX_POSX + i, ctex,
                                       SCE_TRUE, SCE_FALSE) < 0)
            {
                Logger_LogSrc ();
                return SCE_ERROR;
            }
        }
    else if (SCE_CAddRenderTexture (tex->fb[0], id, target,
                                    ctex, SCE_TRUE, SCE_FALSE) < 0)
    {
        Logger_LogSrc ();
        return SCE_ERROR;
    }

    /* on cherche le premier emplacement disponible */
    while (tex->tex[i] != NULL)
        i++;
    tex->tex[i] = ctex;
    tex->canfree[i] = canfree;

    return SCE_OK;
}
/* ajoute le 22/07/2008 */
/**
 * \brief Adds a texture as a new render target's.
 *
 * This function calls SCE_Texture_AddRenderCTexture() like this:
 * SCE_Texture_AddRenderCTexture (\p tex, \p id, SCE_Texture_GetCTexture
 * (\p new), SCE_FALSE)
 */
int SCE_Texture_AddRenderTexture (SCE_STexture *tex, int id,
                                  SCE_STexture *new)
{
    return SCE_Texture_AddRenderCTexture(tex, id, new->tex[new->ct], SCE_FALSE);
}

/* ajoute le 25/08/2008 */
/**
 * \brief Blit a texture over another texture 
 *
 * This function is equivalent to SCE_Texture_Blitf() except that the
 * rectangles specifies pixels instead of scale
 * \sa SCE_Texture_RenderTo() SCE_Texture_Blitf()
 */
void SCE_Texture_Blit (SCE_SIntRect *rdst, SCE_STexture *dst,
                       SCE_SIntRect *rsrc, SCE_STexture *src)
{
    SCE_SFloatRect r1 = {0., 0., 1., 1.};
    SCE_SFloatRect r2 = {0., 0., 1., 1.};
    int w;
    int h;
    if (rdst)
    {
        w = SCE_Texture_GetWidth (dst, 0, 0);
        h = SCE_Texture_GetHeight (dst, 0, 0);
        r1.x1 = (float)rdst->x1 / w;
        r1.y1 = (float)rdst->y1 / h;
        r1.x2 = (float)rdst->x2 / w;
        r1.y2 = (float)rdst->y2 / h;
    }
    if (rsrc)
    {
        w = SCE_Texture_GetWidth (src, 0, 0);
        h = SCE_Texture_GetHeight (src, 0, 0);
        r2.x1 = (float)rsrc->x1 / w;
        r2.y1 = (float)rsrc->y1 / h;
        r2.x2 = (float)rsrc->x2 / w;
        r2.y2 = (float)rsrc->y2 / h;
    }
    SCE_Texture_Blitf (&r1, dst, &r2, src);
}

/* ajoute le 25/08/2008 */
/**
 * \brief Blit a texture over another texture
 * \param rdst the rectangle which defines the area where blit
 * \param dst the texture where blit
 * \param rsrc the rectangle which defines the area to blit
 * \param src the texture to blit
 *
 * If \p dst don't have a frame buffer, a frame buffer is created and added to
 * \p dst, the frame buffer created has only one render target that is
 * SCE_Texture_GetCTexture (\p dst), when calling SCE_Texture_RenderTo() on the
 * frame buffer, none buffer will be cleared (SCE_CClear() isn't called).
 * The rectangles specifies the texture's coordinates between 0 and 1.
 *
 * \sa SCE_Texture_RenderTo() SCE_Texture_Blit() SCE_SFloatRect
 */
void SCE_Texture_Blitf (SCE_SFloatRect *rdst, SCE_STexture *dst,
                        SCE_SFloatRect *rsrc, SCE_STexture *src)
{
    SCE_TMatrix4 mat;
    int w;
    int h;
    if (!dst->fb[0])
    {
        dst->fb[0] = SCE_CCreateFramebuffer ();
        if (!dst->fb[0])
        {
            Logger_LogSrc ();
            return; /* \o/ */
        }
        SCE_CAddRenderTexture (dst->fb[0], SCE_COLOR_BUFFER, 0,
                               dst->tex[dst->ct], 0, !dst->canfree[dst->ct]);
        SCE_CClearRenderbuffer (dst->fb[0], SCE_COLOR_BUFFER, SCE_FALSE);
    }
    w = SCE_Texture_GetWidth (dst, 0, 0);
    h = SCE_Texture_GetHeight (dst, 0, 0);
    SCE_Texture_RenderTo (dst, 0);
    if (rdst)
        SCE_CViewport (rdst->x1 * w, rdst->y1 * h,
                       SCE_Rectangle_GetWidthf (rdst) * w,
                       SCE_Rectangle_GetHeightf (rdst) * h);
    SCE_Texture_Use (src, 0);

    SCE_CSetActiveMatrix (SCE_MAT_TEXTURE);
    SCE_CPushMatrix ();
    if (rsrc)
    {
        SCE_Quad_MakeMatrixFromRectanglef (mat, rsrc);
        SCE_CLoadMatrix (mat);
    }
    else
        SCE_CLoadIdentityMatrix ();
    SCE_CSetActiveMatrix (SCE_MAT_MODELVIEW);
    SCE_CPushMatrix ();
    SCE_CLoadIdentityMatrix ();

    SCE_Quad_Draw (-1., -1., 2., 2.);

    SCE_CPopMatrix ();
    SCE_CSetActiveMatrix (SCE_MAT_TEXTURE);
    SCE_CPopMatrix ();
    SCE_CSetActiveMatrix (SCE_MAT_MODELVIEW);

    SCE_Texture_Use (NULL, 0);
    /* restaure aussi le viewport */
    SCE_Texture_RenderTo (NULL, 0);
}

/* ajoute le 14/06/2007 */
/**
 * \brief Sets the status of the texturing
 * \param enabled is texturing enabled ?
 */
void SCE_Texture_Activate (int enabled)
{
    sce_tex_enabled = enabled;
}
/* ajoute le 14/06/2007 */
/**
 * \brief Enables the texturing
 */
void SCE_Texture_Enable (void)
{
    sce_tex_enabled = SCE_TRUE;
}
/* ajoute le 14/06/2007 */
/**
 * \brief Disables the texturing
 */
void SCE_Texture_Disable (void)
{
    sce_tex_enabled = SCE_FALSE;
}

/* revise le 07/10/2007 */
/**
 * \brief Sets the used texture for the further renders
 * \param tex the texture to use
 * \param unit the texture unit to bind \p tex
 * \sa SCE_CUseTexture()
 *
 * After calling this function, \p tex will be used for the texturing in
 * texture unit \p unit for all the further renders. Set \p tex at NULL
 * to disable the texturing in the texture unit \p unit.
 */
void SCE_Texture_Use (SCE_STexture *tex, unsigned int unit)
{
    if (sce_tex_enabled)
    {
        /* desactivation de la precedente texture */
        SCE_CUseTexture (NULL, unit);

        if (tex)
            SCE_CUseTexture (tex->tex[tex->ct], unit);

        used = tex;
        used_unit = unit;
    }
}


/* ajoute le 04/07/2007 */
/* revise le 07/10/2007 */
/**
 * \brief Uses \p tex instead of the default OpenGL's render buffer
 * \param tex the texture on make the further renders
 * \param cubeface used only for cubemaps, determines on which face of the
 * cubemap the render will be make
 * \sa SCE_CUseFramebuffer()
 * 
 * This function use the default frame buffer of \p tex. If \p tex isn't a
 * render buffer, calling of this function is equivalent of
 * SCE_CUseFramebuffer (NULL, NULL).
 */
void SCE_Texture_RenderTo (SCE_STexture *tex, SCEuint cubeface)
{
    if (tex)
    {
        /* s'il s'agit d'une cubemap, on prend en compte 'cubeface' */
        if (SCE_CGetTextureTarget (tex->tex[tex->ct]) == SCE_TEX_CUBE)
            SCE_CUseFramebuffer (tex->fb[cubeface], NULL);
        else
            SCE_CUseFramebuffer (tex->fb[0], NULL);
    }
    else
        SCE_CUseFramebuffer (NULL, NULL);
}
/* ajoute le 07/10/2007 */
void SCE_Texture_RenderTo_ (SCEuint cubeface)
{
    SCE_Texture_RenderTo (binded, cubeface);
}

/** @} */
